@@ -6,6 +6,8 @@ import requests
 import pandas as pd
 import numpy as np
 from tethysts import Tethys
+from shapely.geometry import mapping
+from shapely import wkb
 
 ######################################
 #### Parameters
@@ -47,10 +49,10 @@ def get_altitude(stn_df, dem_remote=dem_remote, dem_ds_id=dem_ds_id, source_ds_i
     """
 
     """
-    stn_df1 = stn_df[['station_id', 'lon', 'lat']].drop_duplicates(subset=['station_id']).copy()
+    stn_df1 = stn_df[['station_id', 'geometry']].drop_duplicates(subset=['station_id']).copy()
 
     ## Get existing stns
-    if isinstance(source_remote, dict) and isinstance(source_ds_id):
+    if isinstance(source_remote, dict) and isinstance(source_ds_id, str):
         try:
             t1 = Tethys([source_remote])
             old_stns = t1.get_stations(source_ds_id)
@@ -77,9 +79,10 @@ def get_altitude(stn_df, dem_remote=dem_remote, dem_ds_id=dem_ds_id, source_ds_i
 
     alt_list = []
     for i, row in combo1.iterrows():
-        print(i)
+        # print(i)
         if np.isnan(row['altitude']):
-            results = t1.get_results(dem_ds_id, lon=row['lon'], lat=row['lat'], squeeze_dims=True, output='DataArray', cache='memory')
+            geo1 = mapping(wkb.loads(row['geometry'], True))
+            results = t1.get_results(dem_ds_id, lon=float(geo1['coordinates'][0]), lat=float(geo1['coordinates'][1]), squeeze_dims=True, output='DataArray', cache='memory')
             alt = int(results)
         else:
             alt = int(row['altitude'])
