@@ -197,7 +197,7 @@ class Processor(object):
         return dataset_list
 
 
-    def load_raw_grid(self, data_path, lon_dim_name, lat_dim_name, time_dim_name=None, chunks=None, parallel=True, preprocessor=None, postprocessor=None):
+    def load_raw_grid(self, data_path, lon_dim_name, lat_dim_name, time_dim_name=None, chunks=None, parallel=True, preprocessor=None, postprocessor=None, **kwargs):
         """
         The processor takes a str path to one or many files that xr.open_mfdataset can open (e.g. netcdf) and runs a preprocessor and postprocessor (optionally) to provide the proper dimensions and names for the dataset. A Dataset can also be passed to data_path and this will skip the preprocessing step.
         """
@@ -221,7 +221,7 @@ class Processor(object):
             raise TypeError('data_path must be a str path that xr.open_mfdataset can open or an xr.Dataset.')
 
         if isinstance(postprocessor, types.FunctionType):
-            grid1 = postprocessor(grid1)
+            grid1 = postprocessor(grid1, **kwargs)
 
         ## add attributes
         self._dims = grid1.dims
@@ -231,6 +231,7 @@ class Processor(object):
         self._time_dim_name = time_dim_name
         self._preprocessor = preprocessor
         self._postprocessor = postprocessor
+        self._kwargs = kwargs
 
         return grid1
 
@@ -274,7 +275,7 @@ class Processor(object):
         chunks = {self._time_dim_name: self._dims['time'], self._lon_dim_name: xy_chunk_size, self._lat_dim_name: xy_chunk_size}
 
         ## Open grid
-        grid1 = self.load_raw_grid(self._source_data_path, self._lon_dim_name, self._lat_dim_name, self._time_dim_name, chunks=chunks, parallel=True, preprocessor=self._preprocessor, postprocessor=self._postprocessor)
+        grid1 = self.load_raw_grid(self._source_data_path, self._lon_dim_name, self._lat_dim_name, self._time_dim_name, chunks=chunks, parallel=True, preprocessor=self._preprocessor, postprocessor=self._postprocessor, **self._kwargs)
 
         ## Create the blocks
         block_list = split_grid(grid1, x_size=xy_chunk_size, y_size=xy_chunk_size, n_intervals=None, x_name='lon', y_name='lat', mbytes_size=1600)
@@ -328,7 +329,7 @@ class Processor(object):
         else:
             chunks = None
 
-        grid1 = self.load_raw_grid(self._source_data_path, self._lon_dim_name, self._lat_dim_name, self._time_dim_name, chunks=chunks, parallel=True, preprocessor=self._preprocessor, postprocessor=self._postprocessor)
+        grid1 = self.load_raw_grid(self._source_data_path, self._lon_dim_name, self._lat_dim_name, self._time_dim_name, chunks=chunks, parallel=True, preprocessor=self._preprocessor, postprocessor=self._postprocessor, **self._kwargs)
         pts_df = select_sample_points(grid1, samples=samples)
 
         self.sample_points = pts_df
@@ -348,7 +349,7 @@ class Processor(object):
         else:
             chunks = None
 
-        grid1 = self.load_raw_grid(self._source_data_path, self._lon_dim_name, self._lat_dim_name, self._time_dim_name, chunks=chunks, parallel=True, preprocessor=self._preprocessor, postprocessor=self._postprocessor)
+        grid1 = self.load_raw_grid(self._source_data_path, self._lon_dim_name, self._lat_dim_name, self._time_dim_name, chunks=chunks, parallel=True, preprocessor=self._preprocessor, postprocessor=self._postprocessor, **self._kwargs)
 
         grid2 = grid1.isel(time=0, height=0, drop=True).copy()
 
@@ -385,7 +386,7 @@ class Processor(object):
         pts_df = self._select_sample_points(4)
 
         # calc obj sizes
-        grid1 = self.load_raw_grid(self._source_data_path, self._lon_dim_name, self._lat_dim_name, self._time_dim_name, chunks={self._time_dim_name: self._dims['time'], self._lon_dim_name: 1, self._lat_dim_name: 1}, parallel=True, preprocessor=self._preprocessor, postprocessor=self._postprocessor)
+        grid1 = self.load_raw_grid(self._source_data_path, self._lon_dim_name, self._lat_dim_name, self._time_dim_name, chunks={self._time_dim_name: self._dims['time'], self._lon_dim_name: 1, self._lat_dim_name: 1}, parallel=True, preprocessor=self._preprocessor, postprocessor=self._postprocessor, **self._kwargs)
 
         grid2 = grid1[ds_vars]
 
@@ -498,7 +499,7 @@ class Grid(Titan):
         return dataset_list
 
 
-    def load_grid_results(self, results, xy_size, dataset_id=None, sum_closed='right', other_closed='left', discrete=True, other_attrs=None, other_encoding=None, run_date=None, skip_resampling=True):
+    def load_results(self, results, xy_size, dataset_id=None, sum_closed='right', other_closed='left', discrete=True, other_attrs=None, other_encoding=None, run_date=None, skip_resampling=True):
         """
 
         """
@@ -536,26 +537,11 @@ class Grid(Titan):
                 raise TypeError('If paths are not contained in results, then a dataset_id of a str must be passed.')
 
         ## Iterate through the datasets
-        # load_results = self.load_results
-
+        load_results = super().load_results
 
         for ds_id, data_list in data_dict.items():
             print(ds_id)
 
-
-            # def process_data(data):
-            #     data1 = data.copy()
-            #     data1.load()
-            #     data.close()
-            #     data = None
-
-            #     blocks = split_grid(data1, xy_size, xy_size)
-            #     data1.close()
-            #     data1 = None
-
-            #     for block in blocks:
-            #         load_results(block, ds_id, sum_closed=sum_closed, other_closed=other_closed, discrete=discrete, other_attrs=other_attrs, other_encoding=other_encoding, run_date=run_date, ignore_resampling=ignore_resampling)
-
             n_files = len(data_list)
 
             for i, data in enumerate(data_list):
@@ -576,13 +562,7 @@ class Grid(Titan):
                     n_null = max([int(t1[t].values) for t in t1])
 
                     if n_null < total_n_values:
-                        self.load_results(block, ds_id, sum_closed=sum_closed, other_closed=other_closed, discrete=discrete, other_attrs=other_attrs, other_encoding=other_encoding, run_date=run_date, skip_resampling=skip_resampling)
-
-
-            # with concurrent.futures.ProcessPoolExecutor(max_workers=max_workers) as executor:
-            #     results = executor.map(process_data, data_list)
-
-            # self.results_dict[ds_id].extend(results)
+                        load_results(block, ds_id, sum_closed=sum_closed, other_closed=other_closed, discrete=discrete, other_attrs=other_attrs, other_encoding=other_encoding, run_date=run_date, skip_resampling=skip_resampling)
 
 
 
