@@ -601,7 +601,7 @@ def compare_xrs(old_xr, new_xr, add_old=False):
     comp = compare_dfs(old_df, new_df, on, parameter, add_old=add_old)
 
     if comp.empty:
-        print('Nothing has changed. Returning empty DataFrame.')
+        # print('Nothing has changed. Returning empty DataFrame.')
         return comp
 
     else:
@@ -681,7 +681,7 @@ def process_datasets(datasets):
     return dataset_list
 
 
-def create_geometry_df(df, extent=False, altitude=True, to_wkb_hex=False, precision=7):
+def create_geometry_df(df, extent=False, altitude=True, to_wkb_hex=True, precision=7):
     """
 
     """
@@ -698,6 +698,7 @@ def create_geometry_df(df, extent=False, altitude=True, to_wkb_hex=False, precis
                 geometry = shape(geometry).wkb_hex
     else:
         if 'geometry' in df:
+            # TODO: write isinstance check for geopandas or dict
             geometry = df['geometry']
         elif ('lon' in df) and ('lon' in df):
             if altitude:
@@ -730,15 +731,48 @@ def assign_station_ids_df(stns_df, extent=False, precision=5):
     """
 
     """
-    geometry = create_geometry_df(stns_df, extent, False, precision)
+    geometry = create_geometry_df(stns_df, extent=extent, altitude=False, to_wkb_hex=False, precision=precision)
 
     stn_ids = geometry.apply(lambda x: assign_station_id(x))
 
     return stn_ids
 
 
-def process_stations_df(stns, dataset_id=None, remote=None):
+# def process_stations_df(stns, dataset_id=None, remote=None):
+#     """
+
+#     """
+#     ## Get existing stations from tethys
+#     if isinstance(dataset_id, str) and isinstance(remote, dict):
+#         try:
+#             tethys = Tethys([remote])
+#             remote_stns = tethys.get_stations(dataset_id)
+#             remote_stns1 = pd.DataFrame(remote_stns)[['station_id', 'ref']]
+
+#             ## Assign station_ids
+#             stns2 = pd.merge(stns, remote_stns1, on='ref', how='left')
+#         except:
+#             stns2 = stns.copy()
+#             stns2['station_id'] = np.nan
+#     else:
+#         stns2 = stns.copy()
+#         stns2['station_id'] = np.nan
+
+#     stns2.loc[stns2['station_id'].isnull(), 'station_id'] = assign_station_ids_df(stns2[stns2['station_id'].isnull()])
+#     stns2 = stns2.drop_duplicates('station_id').copy()
+
+#     ## Assign geometries
+#     stns2['geometry'] = create_geometry_df(stns2, to_wkb_hex=True, precision=6)
+
+#     ## Final station processing
+#     stns3 = stns2.drop(['lat', 'lon'], axis=1)
+
+#     return stns3
+
+
+def process_sparse_stations_from_df(stns, dataset_id=None, remote=None):
     """
+    Function that takes a stns dataframe of station data and converts it to an Xarray Dataset for Tethys. This is ultimately meant to be combined with the time series data for futher processing. If a geometry column is provided, it must be as a geojson-type dict (not a geopandas column).
 
     """
     ## Get existing stations from tethys
@@ -764,9 +798,11 @@ def process_stations_df(stns, dataset_id=None, remote=None):
     stns2['geometry'] = create_geometry_df(stns2, to_wkb_hex=True, precision=6)
 
     ## Final station processing
-    stns3 = stns2.drop(['lat', 'lon'], axis=1)
+    stns3 = stns2.drop(['lat', 'lon'], axis=1).set_index('geometry')
 
-    return stns3
+    stns4 = stns3.to_xarray()
+
+    return stns4
 
 
 def stations_dict_to_df(stns):
@@ -843,10 +879,8 @@ def get_new_stats(data):
     mean1 = round(float(data1.mean()), precision)
     median1 = round(float(data1.median()), precision)
     count1 = int(data1.count())
-    from_date = pd.Timestamp(data['time'].min().values).tz_localize(None)
-    to_date = pd.Timestamp(data['time'].max().values).tz_localize(None)
 
-    stats1 = Stats(min=min1, max=max1, mean=mean1, median=median1, count=count1, from_date=from_date, to_date=to_date)
+    stats1 = Stats(min=min1, max=max1, mean=mean1, median=median1, count=count1)
 
     return stats1
 
@@ -897,6 +931,17 @@ def get_station_data_from_xr(data):
     #     stn_data1['virtual_station'] = False
 
     stn_data1['dimensions'] = dims0
+    stn_data1['heights'] = data['height'].values.tolist()
+
+    from_date = pd.Timestamp(data['time'].min().values).tz_localize(None)
+    to_date = pd.Timestamp(data['time'].max().values).tz_localize(None)
+
+    stn_data1['time_range'] = {'from_date': from_date, 'to_date': to_date}
+    stn_data1['dataset_id'] = attrs['dataset_id']
+
+    ## get the stats
+    stats1 = get_new_stats(data)
+    stn_data1['stats'] = stats1
 
     if props:
         stn_data1['properties'] = props
@@ -907,7 +952,7 @@ def get_station_data_from_xr(data):
     return stn_m.dict(exclude_none=True)
 
 
-def process_station_summ(dataset_id, station_id, data, object_infos, mod_date=None):
+def process_station_summ(data, object_infos, mod_date=None):
     """
 
     """
@@ -916,13 +961,9 @@ def process_station_summ(dataset_id, station_id, data, object_infos, mod_date=No
     elif isinstance(mod_date, (str, pd.Timestamp)):
         mod_date = pd.Timestamp(mod_date).tz_localize(None)
 
-    ## Generate the info for the recently created data
-    # station_id = str(data['station_id'].values)
-    stats1 = get_new_stats(data)
-
-    ## Put it all together
+    ## Append the obj infos to the other station data
     stn_dict2 = get_station_data_from_xr(data)
-    stn_dict2.update({'dataset_id': dataset_id, 'stats': stats1, 'results_object_key': object_infos, 'modified_date': mod_date})
+    stn_dict2.update({'results_object_key': object_infos, 'modified_date': mod_date})
 
     station_m = Station(**stn_dict2)
 
@@ -1217,14 +1258,20 @@ def stats_for_dataset_metadata(stns):
     extent1 = mapping(box(min_lon, min_lat, max_lon, max_lat))
 
     ## time range
-    trange1 = np.array([[s['stats']['from_date'], s['stats']['to_date']] for s in stns])
+    trange1 = np.array([[s['time_range']['from_date'], s['time_range']['to_date']] for s in stns])
     mins, maxes = trange1.T
 
     min_t = min(mins)
     max_t = max(maxes)
 
+    ## Heights
+    heights1 = []
+    [heights1.extend(s['heights']) for s in stns]
+    heights2 = list(set(heights1))
+    heights2.sort()
+
     ## Make the dict output
-    dict1 = {'spatial_resolution': res1, 'extent': extent1, 'time_range': {'from_date': min_t, 'to_date': max_t}}
+    dict1 = {'spatial_resolution': res1, 'extent': extent1, 'time_range': {'from_date': min_t, 'to_date': max_t}, 'heights': heights2}
 
     return dict1
 
