@@ -10,14 +10,14 @@ import zstandard as zstd
 import pandas as pd
 import xarray as xr
 import orjson
-from tethys_utils.data_models import Geometry, Dataset, DatasetBase, S3ObjectKey, Station, Stats, StationBase
-# from data_models import Geometry, Dataset, DatasetBase, S3ObjectKey, Station, Stats, StationBase
+from tethys_utils.data_models import Dataset, S3ObjectKey, Station
+# from data_models import Geometry, Dataset, DatasetBase, S3ObjectKey
 from multiprocessing.pool import ThreadPool, Pool
 from tethysts.utils import key_patterns, get_object_s3, s3_connection, read_json_zstd, read_pkl_zstd
 from tethys_utils.misc import make_run_date_key, write_json_zstd, write_pkl_zstd
 # from misc import make_run_date_key, write_json_zstd, write_pkl_zstd
 from tethys_utils.processing import compare_xrs, process_station_summ, stats_for_dataset_metadata
-# from processing import compare_xrs, process_station_summ
+# from processing import compare_xrs, process_station_summ, stats_for_dataset_metadata
 from datetime import date, datetime
 
 ############################################
@@ -29,7 +29,7 @@ from datetime import date, datetime
 ### Functions
 
 
-def list_objects_s3(s3_client, bucket, prefix, start_after='', delimiter='', continuation_token='', get_versions=False):
+def list_objects_s3(s3_client, bucket, prefix, start_after='', delimiter='', continuation_token=''):
     """
     Wrapper S3 function around the list_objects_v2 base function with a Pandas DataFrame output.
 
@@ -147,7 +147,7 @@ def list_object_versions_s3(s3_client, bucket, prefix, next_key='', delimiter=No
     return f_df1
 
 
-def put_remote_dataset(s3, bucket, dataset, stns, run_date=None):
+def put_remote_dataset(s3, bucket, dataset, stns, run_date=None, version=3):
     """
 
     """
@@ -164,7 +164,7 @@ def put_remote_dataset(s3, bucket, dataset, stns, run_date=None):
     dataset.update(ds_stats)
 
     ## Add version number
-    dataset.update({'version': 2})
+    dataset.update({'version': version})
 
     ## Check and create dataset metadata
     ds4 = Dataset(**dataset)
@@ -174,31 +174,31 @@ def put_remote_dataset(s3, bucket, dataset, stns, run_date=None):
     ## Write the object
     ds_obj = write_json_zstd(ds5)
 
-    ds_key = key_patterns['dataset'].format(dataset_id=dataset_id)
+    ds_key = key_patterns[version]['dataset'].format(dataset_id=dataset_id)
 
     obj2 = s3.put_object(Bucket=bucket, Key=ds_key, Body=ds_obj, Metadata={'run_date': run_date_key}, ContentType='application/json')
 
     return ds5
 
 
-def get_object_keys(s3, bucket, prefix):
-    """
+# def get_object_keys(s3, bucket, prefix):
+#     """
 
-    """
-    keys1 = list_objects_s3(s3, bucket, prefix)
-    keys2 = keys1[keys1.Key.str.contains('results')]
+#     """
+#     keys1 = list_objects_s3(s3, bucket, prefix)
+#     keys2 = keys1[keys1.Key.str.contains('results')]
 
-    infos1 = [S3ObjectKey(key=row['Key'], bucket=bucket, content_length=row['Size'], etag=row['ETag'], run_date=row['KeyDate'], modified_date=row['LastModified']) for i, row in keys2.iterrows()]
+#     infos1 = [S3ObjectKey(key=row['Key'], bucket=bucket, content_length=row['Size'], etag=row['ETag'], run_date=row['KeyDate'], modified_date=row['LastModified']) for i, row in keys2.iterrows()]
 
-    return infos1
+#     return infos1
 
 
-def get_remote_dataset(s3, bucket, dataset_id=None, ds_key=None):
+def get_remote_dataset(s3, bucket, dataset_id=None, ds_key=None, version=3):
     """
 
     """
     if isinstance(dataset_id, str):
-        ds_key = key_patterns['dataset'].format(dataset_id=dataset_id)
+        ds_key = key_patterns[version]['dataset'].format(dataset_id=dataset_id)
 
     try:
         obj1 = s3.get_object(Bucket=bucket, Key=ds_key)
@@ -211,12 +211,12 @@ def get_remote_dataset(s3, bucket, dataset_id=None, ds_key=None):
     return rem_ds
 
 
-def get_remote_station(s3, bucket, dataset_id=None, station_id=None, stn_key=None):
+def get_remote_station(s3, bucket, dataset_id=None, station_id=None, stn_key=None, version=3):
     """
 
     """
     if isinstance(dataset_id, str):
-        stn_key = key_patterns['station'].format(dataset_id=dataset_id, station_id=station_id)
+        stn_key = key_patterns[version]['station'].format(dataset_id=dataset_id, station_id=station_id)
 
     try:
         obj1 = s3.get_object(Bucket=bucket, Key=stn_key)
@@ -229,7 +229,7 @@ def get_remote_station(s3, bucket, dataset_id=None, station_id=None, stn_key=Non
     return rem_stn
 
 
-def put_remote_station(s3, bucket, station, run_date=None):
+def put_remote_station(s3, bucket, station, run_date=None, version=3):
     """
 
     """
@@ -244,79 +244,95 @@ def put_remote_station(s3, bucket, station, run_date=None):
 
     stn_obj = write_json_zstd(stn5)
 
-    stn_key = key_patterns['station'].format(dataset_id=dataset_id, station_id=station_id)
+    stn_key = key_patterns[version]['station'].format(dataset_id=dataset_id, station_id=station_id)
 
     obj2 = s3.put_object(Bucket=bucket, Key=stn_key, Body=stn_obj, Metadata={'run_date': run_date_key}, ContentType='application/json')
 
     return stn5
 
 
-def put_remote_agg_stations(s3, bucket, dataset_id, threads=30):
+def put_remote_agg_stations(s3, bucket, dataset_id, threads=30, version=3):
     """
 
     """
-    # base_stn_key = key_patterns['station']
-    agg_stn_key = key_patterns['stations']
+    stn_key = key_patterns[version]['station']
 
-    stn_prefix = agg_stn_key.split('stations.json.zst')[0].format(dataset_id=dataset_id)
+    stn_prefix = stn_key.split('{station_id}')[0].format(dataset_id=dataset_id)
+
+    if version == 2:
+        list1 = list_objects_s3(s3, bucket, stn_prefix)
+    elif version == 3:
+        list1 = list_objects_s3(s3, bucket, stn_prefix, delimiter='/')
+    else:
+        raise ValueError('Wrong object structure version.')
 
-    list1 = list_objects_s3(s3, bucket, stn_prefix)
     list2 = list1[list1.Key.str.contains('station.json.zst')].copy()
 
-    # stn_list = [{'s3': s3, 'bucket': bucket, 'dataset_id': None, 'stn_key': k} for k in list2.Key]
-    stn_list = [[s3, bucket, None, None, k] for k in list2.Key]
+    if not list2.empty:
 
-    output = ThreadPool(threads).starmap(get_remote_station, stn_list)
+        stn_list = [[s3, bucket, None, None, k, version] for k in list2.Key]
 
-    ## Separate the results obj keys from the stations
-    results_obj_keys = [{'dataset_id': s['dataset_id'], 'station_id': s['station_id'], 'results_object_key': s['results_object_key']} for s in output]
-    [s.update({'results_object_key': s['results_object_key'][-1]}) for s in output]
+        output = ThreadPool(threads).starmap(get_remote_station, stn_list)
 
-    ## Save files
-    # Stations
-    stns_obj = write_json_zstd(output)
+        ## Separate the results obj keys from the stations
+        results_obj_keys = [{'dataset_id': s['dataset_id'], 'station_id': s['station_id'], 'results_object_key': s['results_object_key']} for s in output]
+        [s.update({'results_object_key': s['results_object_key'][-1]}) for s in output]
 
-    run_date_key = make_run_date_key()
-    stns_key = agg_stn_key.format(dataset_id=dataset_id)
-    s3.put_object(Body=stns_obj, Bucket=bucket, Key=stns_key, ContentType='application/json', Metadata={'run_date': run_date_key})
+        ## Save files
+        # Stations
+        stns_obj = write_json_zstd(output)
 
-    # Results object keys
-    ro_key = key_patterns['results_object_keys'].format(dataset_id=dataset_id)
+        run_date_key = make_run_date_key()
 
-    ro_obj = write_json_zstd(results_obj_keys)
+        agg_stn_key = key_patterns[version]['stations']
 
-    s3.put_object(Body=ro_obj, Bucket=bucket, Key=ro_key, ContentType='application/json', Metadata={'run_date': run_date_key})
+        stns_key = agg_stn_key.format(dataset_id=dataset_id)
+        s3.put_object(Body=stns_obj, Bucket=bucket, Key=stns_key, ContentType='application/json', Metadata={'run_date': run_date_key})
 
-    return output
+        # Results object keys
+        ro_key = key_patterns[version]['results_object_keys'].format(dataset_id=dataset_id)
+
+        ro_obj = write_json_zstd(results_obj_keys)
+
+        s3.put_object(Body=ro_obj, Bucket=bucket, Key=ro_key, ContentType='application/json', Metadata={'run_date': run_date_key})
+
+        return output
+    else:
+        print('There are no stations files in the database.')
+        return None
 
 
-def put_remote_agg_datasets(s3, bucket, threads=30):
+def put_remote_agg_datasets(s3, bucket, threads=30, version=3):
     """
 
     """
-    # base_ds_key = key_patterns['dataset']
-    agg_ds_key = key_patterns['datasets']
+    ds_key = key_patterns[version]['dataset']
 
-    ds_prefix = agg_ds_key.split('datasets.json.zst')[0]
+    ds_prefix = ds_key.split('{dataset_id}')[0]
+
+    if version == 2:
+        list1 = list_objects_s3(s3, bucket, ds_prefix)
+    elif version == 3:
+        list1 = list_objects_s3(s3, bucket, ds_prefix, delimiter='/')
+    else:
+        raise ValueError('Wrong object structure version.')
 
-    list1 = list_objects_s3(s3, bucket, ds_prefix)
     list2 = list1[list1.Key.str.contains('dataset.json.zst')].copy()
 
-    # ds_list = [{'s3': s3, 'bucket': bucket, 'dataset_id': None, 'ds_key': k} for k in list2.Key]
-    ds_list = [[s3, bucket, None, k] for k in list2.Key]
+    ds_list = [[s3, bucket, None, k, version] for k in list2.Key]
 
     output = ThreadPool(threads).starmap(get_remote_dataset, ds_list)
 
     dss_obj = write_json_zstd(output)
 
     run_date_key = make_run_date_key()
-    dss_key = agg_ds_key
+    dss_key = key_patterns[version]['datasets']
     s3.put_object(Body=dss_obj, Bucket=bucket, Key=dss_key, ContentType='application/json', Metadata={'run_date': run_date_key})
 
     return output
 
 
-def compare_datasets_from_s3(conn_config, bucket, new_data, add_old=False, last_run_date_key=None, public_url=None):
+def compare_datasets_from_s3(conn_config, bucket, new_data, add_old=False, last_run_date_key=None, public_url=None, version=3):
     """
     Parameters
     ----------
@@ -347,7 +363,7 @@ def compare_datasets_from_s3(conn_config, bucket, new_data, add_old=False, last_
 
     key_dict = {'dataset_id': dataset_id, 'station_id': station_id}
 
-    base_key_pattern = key_patterns['results']
+    base_key_pattern = key_patterns[version]['results']
 
     ## Get list of keys
     if isinstance(last_run_date_key, str):
@@ -373,7 +389,7 @@ def compare_datasets_from_s3(conn_config, bucket, new_data, add_old=False, last_
     return up1
 
 
-def update_results_s3(processing_code, data_dict, run_date_dict, remote, threads=30, public_url=None):
+def update_results_s3(processing_code, data_dict, run_date_dict, remote, threads=30, public_url=None, version=3):
     """
     Parameters
     ----------
@@ -423,7 +439,7 @@ def update_results_s3(processing_code, data_dict, run_date_dict, remote, threads
         run_date_key = run_date_dict[ds_id]
 
         # Create the Key info dict
-        prefix = key_patterns['results'].split('{station_id}')[0].format(dataset_id=ds_id)
+        prefix = key_patterns[version]['results'].split('{station_id}')[0].format(dataset_id=ds_id)
 
         keys1 = list_objects_s3(s3, bucket, prefix)
         obj_df1 = keys1[keys1.Key.str.contains('results.nc')].copy()
@@ -463,7 +479,7 @@ def update_results_s3(processing_code, data_dict, run_date_dict, remote, threads
                 else:
                     last_date_key = make_run_date_key(last_date_key_df['KeyDate'].iloc[0])
 
-                up1 = compare_datasets_from_s3(conn_config, bucket, new1, add_old=add_old, last_run_date_key=last_date_key, public_url=public_url)
+                up1 = compare_datasets_from_s3(conn_config, bucket, new1, add_old=add_old, last_run_date_key=last_date_key, public_url=public_url, version=version)
 
             ## Save results
             if isinstance(up1, xr.Dataset) and (len(up1[parameter].time) > 0):
@@ -471,7 +487,7 @@ def update_results_s3(processing_code, data_dict, run_date_dict, remote, threads
                 # print('Save results')
                 key_dict = {'dataset_id': ds_id, 'station_id': stn_id, 'run_date': run_date_key}
 
-                new_key = key_patterns['results'].format(**key_dict)
+                new_key = key_patterns[version]['results'].format(**key_dict)
 
                 cctx = zstd.ZstdCompressor(level=1)
                 c_obj = cctx.compress(up1.to_netcdf())
@@ -494,9 +510,9 @@ def update_results_s3(processing_code, data_dict, run_date_dict, remote, threads
                 info1 = [S3ObjectKey(key=row['Key'], bucket=bucket, content_length=row['Size'], etag=row['ETag'], run_date=row['KeyDate']) for i, row in stn_obj_df1.iterrows()]
 
                 ## Final station processing and saving
-                stn_m = process_station_summ(ds_id, stn_id, up1, info1, mod_date=mod_date_key)
+                stn_m = process_station_summ(up1, info1, mod_date=mod_date_key)
 
-                up_stns = put_remote_station(s3, bucket, stn_m, run_date=mod_date_key)
+                up_stns = put_remote_station(s3, bucket, stn_m, run_date=mod_date_key, version=version)
 
             else:
                 print('No new data to update')
@@ -514,7 +530,7 @@ def update_results_s3(processing_code, data_dict, run_date_dict, remote, threads
             pool.join()
 
 
-def delete_result_objects_s3(conn_config, bucket, dataset_ids=None, keep_last=10, threads=50):
+def delete_result_objects_s3(conn_config, bucket, dataset_ids=None, keep_last=10, threads=50, version=3):
     """
     Function to delete Tethys result objects including all object versions.
 
@@ -540,7 +556,7 @@ def delete_result_objects_s3(conn_config, bucket, dataset_ids=None, keep_last=10
     if isinstance(dataset_ids, str):
         dataset_ids = [dataset_ids]
 
-    prefix = key_patterns['results'].split('{dataset_id}')[0]
+    prefix = key_patterns[version]['results'].split('{dataset_id}')[0]
 
     obj_list = list_object_versions_s3(s3, bucket, prefix)
 
@@ -583,7 +599,7 @@ def delete_result_objects_s3(conn_config, bucket, dataset_ids=None, keep_last=10
     return rem_keys
 
 
-def process_run_date(processing_code, dataset_list, remote, run_date=None, save_interval_hours=336):
+def process_run_date(processing_code, dataset_list, connection_config, bucket, run_date=None, save_interval_hours=336, version=3):
     """
     Function to process the run date keys for all datasets for the extraction. These are specific to each processing_code.
 
@@ -613,35 +629,33 @@ def process_run_date(processing_code, dataset_list, remote, run_date=None, save_
     else:
         run_date1 = pd.Timestamp.today(tz='utc').tz_localize(None)
 
-    # run_date_key = run_date1.strftime('%Y%m%dT%H%M%SZ')
-
-    s3 = s3_connection(remote['connection_config'])
-
     run_date_dict = {}
     for ds in dataset_list:
         dataset_id = ds['dataset_id']
 
         ## Get last result dates
-        prefix = key_patterns['results'].split('{station_id}')[0].format(dataset_id=dataset_id)
-        obj_list = list_objects_s3(s3, remote['bucket'], prefix)
-
-        if obj_list.empty:
+        try:
+            key1 = key_patterns[version]['stations'].format(dataset_id=dataset_id)
+            obj1 = get_object_s3(key1, connection_config, bucket, counter=0)
+            rok1 = read_json_zstd(obj1)
+            rok2 = pd.DataFrame([r['results_object_key'] for r in rok1])
+            rok2['run_date'] = pd.to_datetime(rok2['run_date'])
+            obj_list = rok2.dropna(subset=['run_date'])
+            last_run_date = obj_list['run_date'].max()
+        except:
             last_run_date = run_date1
-            last_run_date_key = run_date1.strftime('%Y%m%dT%H%M%SZ')
-        else:
-            last_run_date = obj_list['KeyDate'].max()
 
-            if processing_code in [3]:
-                last_run_date_key = last_run_date.strftime('%Y%m%dT%H%M%SZ')
-            elif processing_code in [4, 5]:
+        if processing_code in [3]:
+            last_run_date_key = last_run_date.strftime('%Y%m%dT%H%M%SZ')
+        elif processing_code in [4, 5]:
+            last_run_date_key = run_date1.strftime('%Y%m%dT%H%M%SZ')
+        elif processing_code in [1, 2, 6]:
+            if last_run_date < (run_date1 - pd.DateOffset(hours=save_interval_hours) + pd.DateOffset(minutes=5)):
                 last_run_date_key = run_date1.strftime('%Y%m%dT%H%M%SZ')
-            elif processing_code in [1, 2, 6]:
-                if last_run_date < (run_date1 - pd.DateOffset(hours=save_interval_hours) + pd.DateOffset(minutes=5)):
-                    last_run_date_key = run_date1.strftime('%Y%m%dT%H%M%SZ')
-                else:
-                    last_run_date_key = last_run_date.strftime('%Y%m%dT%H%M%SZ')
             else:
-                raise ValueError('processing_code does not exist.')
+                last_run_date_key = last_run_date.strftime('%Y%m%dT%H%M%SZ')
+        else:
+            raise ValueError('processing_code does not exist.')
 
         run_date_dict.update({dataset_id: last_run_date_key})
 
