@@ -33,6 +33,17 @@ class Titan(object):
         pass
 
 
+    @staticmethod
+    def process_sparse_stations_from_df(stns, dataset_id=None, remote=None):
+        """
+        Function that takes a stns dataframe of station data and converts it to an Xarray Dataset for Tethys. This is ultimately meant to be combined with the time series data for futher processing. If a geometry column is provided, it must be as a geojson-type dict (not a geopandas column).
+
+        """
+        stns = tu.processing.process_sparse_stations_from_df(stns, dataset_id, remote)
+
+        return stns
+
+
     def init_results_dict(self):
         """
 
@@ -56,11 +67,12 @@ class Titan(object):
         self.init_results_dict()
 
 
-    def load_connection_params(self, connection_config, bucket, public_url=None):
+    def load_connection_params(self, connection_config, bucket, public_url=None, version=3):
         """
 
         """
         ## Test S3 connection
+        _ = tu.data_models.Remote(**{'connection_config': connection_config, 'bucket': bucket})
         s3 = s3_connection(connection_config)
 
         try:
@@ -76,15 +88,21 @@ class Titan(object):
         setattr(self, 'connection_config', connection_config)
         setattr(self, 'bucket', bucket)
         setattr(self, 'public_url', public_url)
+        setattr(self, 'version', version)
 
 
-    def load_run_date(self, processing_code, run_date=None):
+    def load_run_date(self, processing_code, run_date=None, save_interval_hours=336):
         """
 
         """
-        remote = {'connection_config': self.connection_config, 'bucket': self.bucket}
+        # remote = {'connection_config': self.connection_config, 'bucket': self.bucket}
 
-        run_date_dict = tu.s3.process_run_date(processing_code, self.dataset_list, remote, run_date)
+        if isinstance(self.public_url, str):
+            conn_config = self.public_url
+        else:
+            conn_config = self.connection_config
+
+        run_date_dict = tu.s3.process_run_date(processing_code, self.dataset_list, conn_config, self.bucket, run_date, save_interval_hours=save_interval_hours, version=self.version)
         max_run_date_key = max(list(run_date_dict.values()))
 
         setattr(self, 'processing_code', processing_code)
@@ -119,7 +137,7 @@ class Titan(object):
         """
         remote = {'connection_config': self.connection_config, 'bucket': self.bucket}
 
-        tu.s3.update_results_s3(self.processing_code, self.results_dict, self.run_date_dict, remote, threads=threads, public_url=self.public_url)
+        tu.s3.update_results_s3(self.processing_code, self.results_dict, self.run_date_dict, remote, threads=threads, public_url=self.public_url, version=self.version)
 
         self.init_results_dict()
 
@@ -133,11 +151,12 @@ class Titan(object):
         s3 = s3_connection(self.connection_config, threads)
 
         for ds in self.dataset_list:
-            ds_stations = tu.s3.put_remote_agg_stations(s3, self.bucket, ds['dataset_id'], threads)
-            ds_new = tu.s3.put_remote_dataset(s3, self.bucket, ds, ds_stations)
+            ds_stations = tu.s3.put_remote_agg_stations(s3, self.bucket, ds['dataset_id'], threads, version=self.version)
+            if ds_stations is not None:
+                ds_new = tu.s3.put_remote_dataset(s3, self.bucket, ds, ds_stations, version=self.version)
 
         # Aggregate all datasets for the bucket
-        ds_all = tu.s3.put_remote_agg_datasets(s3, self.bucket, threads)
+        ds_all = tu.s3.put_remote_agg_datasets(s3, self.bucket, threads, version=self.version)
 
         print('Updating the aggregates has been successful!')
 
